/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include "aidge/graphmatching/Match.hpp"

namespace py = pybind11;
namespace Aidge {
void init_Match(py::module& m){
    py::class_<Match, std::shared_ptr<Match>>(m, "Match", "Match class stores the matched patterns resulting from a graph matching query. A matched pattern is the combinaison of the graph pattern start nodes and the set of all the nodes in the matched pattern (including the start nodes)")
    .def(py::init<>())
    .def("get_nb_match", &Match::getNbMatch, R"mydelimiter(
    :returns: The number of graph patterns matched
    :rtype: int
    )mydelimiter")
    .def("get_start_nodes", &Match::getStartNodes, R"mydelimiter(
    :returns: All matched graph patterns start nodes
    :rtype: List[List[:py:class:`aidge_core.Nodes`]]
    )mydelimiter")
    .def("get_match_nodes", &Match::getMatchNodes, R"mydelimiter(
    :returns: All matched graph patterns sets of matched nodes
    :rtype: List[Set[:py:class:`aidge_core.Nodes`]]
    )mydelimiter");
}
}
