/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/Operator.hpp"
#include <pybind11/stl.h>

namespace py = pybind11;
namespace Aidge {
void init_Operator(py::module& m){
    py::class_<Operator, std::shared_ptr<Operator>>(m, "Operator")
    .def("output", &Operator::output, py::arg("outputIdx"))
    .def("input", &Operator::input, py::arg("inputIdx"))
    .def("nb_data_inputs", &Operator::nbDataInputs)
    .def("associate_input", &Operator::associateInput, py::arg("inputIdx"), py::arg("data"))
    .def("set_datatype", &Operator::setDatatype, py::arg("datatype"))
    .def("set_backend", &Operator::setBackend, py::arg("name"))
    .def("forward", &Operator::forward)
    // py::keep_alive forbide Python to garbage collect implementation will the Operator is not garbade collected !
    .def("set_impl", &Operator::setImpl, py::arg("implementation"), py::keep_alive<1, 2>())
    ;
}
}
