/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>

#include "aidge/utils/Types.h"
// #include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/Operator.hpp"
#include "aidge/operator/Producer.hpp"
#include "aidge/data/Tensor.hpp"

namespace py = pybind11;
namespace Aidge {

template <DimIdx_t DIM>
void declare_Producer(py::module &m) {
    // m.def(("Producer_" + std::to_string(DIM)+"D").c_str(), py::overload_cast<shared_ptr<Node>&>(&Producer<DIM>), py::arg("dims"), py::arg("name"));
    m.def("Producer", static_cast<std::shared_ptr<Node>(*)(const std::array<DimSize_t, DIM>&, const std::string&)>(&Producer), py::arg("dims"), py::arg("name") = "");

}


void init_Producer(py::module &m) {
    py::class_<Producer_Op,  std::shared_ptr<Producer_Op>, Operator>(
        m,
        "ProducerOp",
        py::multiple_inheritance())
    .def("dims", &Producer_Op::dims)
    .def("set_output_tensor", &Producer_Op::setOutputTensor)
    .def("get_inputs_name", &Producer_Op::getInputsName)
    .def("get_outputs_name", &Producer_Op::getOutputsName);
    m.def("Producer", static_cast<std::shared_ptr<Node>(*)(const std::shared_ptr<Tensor>, const std::string&)>(&Producer), py::arg("tensor"), py::arg("name") = "");

    declare_Producer<1>(m);
    declare_Producer<2>(m);
    declare_Producer<3>(m);
    declare_Producer<4>(m);
    declare_Producer<5>(m);
    declare_Producer<6>(m);

    // m.def(("Producer_" + std::to_string(DIM)+"D").c_str(), py::overload_cast<shared_ptr<Node>&>(&Producer<DIM>), py::arg("dims"), py::arg("name"));
}
}
