#include "aidge/graphRegex/matchFsm/MatchResult.hpp"

using namespace Aidge; 


MatchResult::MatchResult(std::vector<std::shared_ptr<FsmRunTimeContext>> allValid, std::size_t nbSubStm):mIdToRunTime(nbSubStm){
        mAllValid = allValid;
        mNbSubStm = nbSubStm;

        //mIdToRunTimm
        for (const auto& contextPtr : allValid) {
            mIdToRunTime[contextPtr->getSubStmId()].push_back(contextPtr);
        }

        std::vector<std::shared_ptr<FsmRunTimeContext>> precedence;
        //make all solution posible 
        _generateCombinationd(0,precedence);
        //sort by solution number of elements
        std::sort(mSolve.begin(), mSolve.end(), [](const std::set<NodePtr>& set1, const std::set<NodePtr>& set2) {
        return set1.size() < set2.size();
        });


}

void MatchResult::_generateCombinationd( std::size_t idxSubStm, std::vector<std::shared_ptr<FsmRunTimeContext>>& precedence){

    //it's end , we are below the number of stm
    if (idxSubStm == mNbSubStm)
    {
        //precedence containe a liste of FSM compatible, we just need to 
        //check if all the node have been valide by at least one contetext 
        
        //1) make the set of all node for the comput graph that are valide in all the  FsmRunTimeContext
        std::set<NodePtr> validNode;
        std::set<NodePtr> rejectNode;
        for (const auto& contextPtr : precedence) {
            std::set<NodePtr> tmpV =  contextPtr->getValidNodes();
            validNode.insert(tmpV.begin(), tmpV.end());
            std::set<NodePtr> tmpR =  contextPtr->getRejectedNodes();
            rejectNode.insert(tmpR.begin(),tmpR.end());
        }
        // 2) all  RejectedNodes need to be valide by an others stm 
        // if it's not the case the match is not valid
        if(std::includes(validNode.begin(), validNode.end(), rejectNode.begin(), rejectNode.end())){
            //we can save the solution 
            mSolve.push_back(validNode);
        }
        precedence.pop_back();
        return;
    }


    for (const auto& contextPtrOneFsm : mIdToRunTime[idxSubStm]) 
    {
        if(idxSubStm == 0){
            precedence.push_back(contextPtrOneFsm);
            _generateCombinationd(idxSubStm+1,precedence);
            
        }else{
            //test if the new context is compatible whith all the context in the precedence
            //
            bool compatibleSolutionFsm = true;
            for (const auto& contextPtrOfOtherFsm : precedence) {
                if(!(contextPtrOneFsm->areCompatible(contextPtrOfOtherFsm))){
                    compatibleSolutionFsm = false;
                    break;
                }
            }

            if(compatibleSolutionFsm){
                precedence.push_back(contextPtrOneFsm);
                _generateCombinationd(idxSubStm+1,precedence);
            }

        }
    }

    if(idxSubStm != 0){
        precedence.pop_back();
    }
    return;

}

std::set<NodePtr> MatchResult::getBiggerSolution(void){
    if(mSolve.empty()){
        return std::set<NodePtr>();
    }else{
        return mSolve[0];
    }
    
}