
#include <catch2/catch_test_macros.hpp>
#include "aidge/nodeTester/ConditionalInterpreter.hpp"
#include "aidge/operator/GenericOperator.hpp"


using namespace Aidge;



TEST_CASE("ConditionalInterpreter", "ConditionalInterpreter") {

    SECTION("custom Lambda") {

        const std::string test = " !toto($) == true " ;
        ConditionalInterpreter conditionalParser = ConditionalInterpreter(test);
        conditionalParser.insertLambda("toto",+[](NodePtr NodeOp){return false;});
        std::shared_ptr<Node> nodeOp = GenericOperator("conv", 0, 0, 0, "Gop1");

        bool result = conditionalParser.test(nodeOp);
        REQUIRE(result == true);
    }

    SECTION("syntax error") {

        const std::string test = "'A' == 'A' ,&& ";
        REQUIRE_THROWS_AS( ConditionalInterpreter(test), std::runtime_error);
  
    }


    SECTION("test false int ") {

        const std::string test = " 10 == 11 " ;
        ConditionalInterpreter conditionalParser = ConditionalInterpreter(test);
        std::shared_ptr<Node> nodeOp = GenericOperator("conv", 0, 0, 0, "Gop1");
        bool result = conditionalParser.test(nodeOp);
        REQUIRE(result == false);
    }

    SECTION("test true int ") {
        const std::string test = " 42 == 42 " ;
        ConditionalInterpreter conditionalParser = ConditionalInterpreter(test);
        std::shared_ptr<Node> nodeOp = GenericOperator("conv", 0, 0, 0, "Gop1");
        bool result = conditionalParser.test(nodeOp);
        REQUIRE(result == true);
    }
    
    SECTION("test false str ") {
        const std::string test = " 'toto' == 'Corgi' " ;
        ConditionalInterpreter conditionalParser = ConditionalInterpreter(test);
        std::shared_ptr<Node> nodeOp = GenericOperator("conv", 0, 0, 0, "Gop1");
        bool result = conditionalParser.test(nodeOp);
        REQUIRE(result == false);
    }

    SECTION("test true str ") {

        const std::string test = " 'Corgi' == 'Corgi' " ;
        ConditionalInterpreter conditionalParser = ConditionalInterpreter(test);
        std::shared_ptr<Node> nodeOp = GenericOperator("conv", 0, 0, 0, "Gop1");
        bool result = conditionalParser.test(nodeOp);
        REQUIRE(result == true);
    }

}