/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <catch2/catch_test_macros.hpp>

#include "aidge/operator/MetaOperator.hpp"
#include "aidge/operator/MetaOperatorDefs.hpp"
#include "aidge/graph/GraphView.hpp"
#include <cstddef>

using namespace Aidge;

TEST_CASE("[core/operators] MetaOperator", "[Operator]") {
    SECTION("PaddedConv") {
        auto op = PaddedConv(1, 3, {3, 3}, "padded_conv", {1, 1}, {1, 1, 1, 1});

        auto microGraph = std::dynamic_pointer_cast<MetaOperator_Op>(op->getOperator())->getMicroGraph();

        REQUIRE(microGraph->getNodes().size() == 2);
        REQUIRE(microGraph->inputNodes().size() == 2);  // 2 because Conv has inputs outside the meta-op (Producers for weight and bias)
        // Order not garanteed by the GraphView
        //REQUIRE((*microGraph->inputNodes().begin())->getOperator()->type() == "Pad");
        REQUIRE(microGraph->outputNodes().size() == 1);
        REQUIRE((*microGraph->outputNodes().begin())->getOperator()->type() == "Conv");
        REQUIRE(op->nbInputs() == 3);
        REQUIRE(op->nbDataInputs() == 1);
        REQUIRE(op->nbOutputs() == 1);

        std::shared_ptr<Tensor> myInput = std::make_shared<Tensor>();
        myInput->resize({2,3,5,5});
        op->getOperator()->associateInput(0,myInput);
        op->getOperator()->computeOutputDims();

        REQUIRE(op->getOperator()->outputDimsForwarded());
        REQUIRE(op->getOperator()->getOutput(0)->dims() == std::vector<size_t>({2,3,5,5}));
        REQUIRE(op->getOperator()->getInput(0) == myInput);
        // Order not garanteed by the GraphView
        //REQUIRE((*microGraph->inputNodes().begin())->getOperator()->getInput(0) == myInput);
        REQUIRE(op->getOperator()->getOutput(0) == (*microGraph->outputNodes().begin())->getOperator()->getOutput(0));
        
        //op->getOperator()->updateConsummerProducer();  // require implementation
        //auto microGraphScheduler = std::dynamic_pointer_cast<MetaOperator_Op>(op->getOperator())->getMicroGraphScheduler();
        //REQUIRE(microGraphScheduler->getStaticScheduling().size() == 2);
    }
}
